
/**************************************************************************************************************/
/* To run SSG1to3_sas via MARCOSSG note:                                                                      */
/* 3-stage selection via IML_SAS requires so much computing time that these calculations are done in R, so it */
/* it is needed to install R on your computer to run the SAS MACROSSG via a SAS input file                    */
/* and to enable SAS via setting in the file "sasv9.cfg" to RUN R comandsin SAS                               */
/* To download and install R go to https://cran.r-project.org                                                 */
/* For more information about the subsequent steps                                                            */
/* https://communities.sas.com/t5/General-SAS-Programming/Run-R-code-inside-SAS-easily/td-p/210116            */
/* For general information                                                                                    */
/* http://go.documentation.sas.com/?docsetId=imlug&docsetTarget=imlug_r_toc.htm&docsetVersion=14.2&locale=en  */
/**************************************************************************************************************/

/**************************************************************************************************************/ 
/* Copy three SAS files on your computer to run SSG1to3_sas:                                                  */
/* 1) file "README_SSG1to3_SAS(1.0).sas with 8 Examples"                                                      */
/* 2) file "macroSSG.sas                                                                                      */
/* 3) file "Rcode.sas"                                                                                        */
/* Suggestion copy these files into desktop, folder SAS_MACROS, and subfolder SSG1to3                         */
/* in the example case C:\Users\WGRUNEBERG\OneDrive - CGIAR\Desktop\SAS_MACROS\SSG1to3                        */                                      
/**************************************************************************************************************/

/**************************************************************************************************************/ 
/* Setting SAS to RUN R comands - BEGIN                                                                       */
/* 1) Locate and open in Notepad or Notepad++ the file "sasv9.cfg"                                            */
/*    In our case for SAS 9.4 in windows OS the file is allocated in:                                         */
/*    C:\Program Files\SASHome\x86\SASFoundation\9.4\sasv9.cfg                                                */
/*    a backup of the original file "sasv9.cfg" is recommended                                                */
/*    often it is required to get adminstration rights to change this file - contact adminstrator if required */                                  
/* 2) Add this 3 lines of command in the end "sasv9.cfg" file (after that save)                               */
/*    it can be required to change your "sasv9.cfg" location, according your SAS version                      */
/*    it can be required to change your R location in "R_HOME", according your R version                      */ 
/*    Settings in "sasv9.cfg":                                                                                */ 
/*    -RLANG                                                                                                  */
/*    -config "C:\Program Files\SASHome\SASFoundation\9.4\nls\en\sasv9.cfg"                                   */
/*    -SET R_HOME "C:\Program Files\R\R-3.1.3"                                                                */
/*                                                                                                            */
/*    Setting SAS to RUN R comands - END                                                                      */
/**************************************************************************************************************/
 
/**************************************************************************************************************/
/* Open SAS file README_SSG1to3_SAS(1.0).sas with 8 Examples.sas and move into the SAS editor:                */
/* the first statements:                                                                                      */ 
/* proc options option=rlang;                                                                                 */
/* run;                                                                                                       */ 
/* options mcompilenote=ALL mprint;                                                                           */
/*                                                                                                            */ 
/* are required and test if R codes are working                                                               */                                                                                                                           
/* by runing these statements                                                                                 */
/* in SAS log must appear the message (or similar message:                                                    */
/* SAS (r) Proprietary Software Release 9.4  TS1M2                                                            */
/* RLANG             Enables SAS to execute R language statements                                             */
/* NOTE: PROCEDURE OPTIONS used (Total process time):                                                         */
/*       real time           0.01 seconds                                                                     */
/*       cpu time            0.01 seconds                                                                     */
/*                                                                                                            */
/* Recommendation first check if you can run R inside SAS                                                     */
/**************************************************************************************************************/

proc options option=rlang;
run;
options mcompilenote=ALL mprint; /*Testing if R codes is working*/

/* General comments for macroSSG.sas containing SSG1to3
 * Readme for data input into the "SSG1to3_SAS" Program
 * The Stage Selection Gain 1to3 (SSG1to3) in SSG1to3 is a simple program to determine the response to selection 
 * in standardized units (R) after 1-stage selection  with up to three seasons of testing, 
 * 2-stage selection with two seasons of testing, and 3-stage selection with three seasons of testing. 
 * No optimization is conducted single selection senarios are tested 
 * The required input for SSG1to3 is a vector of 17 numbers in a typical tempoary SAS DATA input file (with one data line):
 * A qualifier for 1, 2, or 3 stage selection (1, 2, or 3), with 1 for 1-stage selection, 2 for 2-stage selection, 
 * and 3 for 3-stage selection,
 * the total test capacity as number of plots for the entire multiple environmental trial series (for example 540, 1080, 
 * or 2268),
 * the number of finally selected clones - in case of later breeding stages we recommend 2, 3, 4, or 5,
 * the variance component ratios for s_G^2, s_GxL^2, s_GxS^2, s_GxLxS^2, and s_e^2, relative to s_G^2, 
 * where G denotes genotype, L is location, and S is season. The first number for s_G^2 must be 1 and 
 * all other numbers must be entered relative to the variance component estimate s_G^2 from the analysis
 * of variance (for example 1, 0.189, 0.103, 0.603, 1.162),
 * three numbers for the number of clones tested in each season, in case you test no genotypes 
 * at season 3 then set the 3rd number to . (for example 42 42 .) or no genotypes at season 2 and 3 then
 * set the 2nd and 3rd number to . (for example 42 . .)
 * three numbers for the number of locations at each season, in case you test no locations at season 3 then 
 * set 3rd number to . (for example 3 12 .) or no locations at season 2 and 3 then set the 2nd and 3rd number 
 * to . (for example 6 . .),
 * three numbers for the plot replications at each season, in case you test no locations at season 3 set 3rd number to .
 * (for example 1 3 .) or no locations at season 2 and 3 then set the 2nd and 3rd number to . (for example 3 . .)
 * 
 * Results:
 * is given in Results Viewer - SAS Output with your input and output with repeat of input and the response to selection
 * for your allocation input in column labled response 
 * in case the response to selection is not given the plot capacity might be too low for this combination of replications,
 * locations and genotypes - check in SAS Log
 */
 

/* Create dataset with input paramaters
   Example 1: SSG1to3_SAS with input 1 2268 3 1 0.189 0.103 0.603 1.162 42 42 42 9 9 9 2 2 2
   A one stage selection after three seasons of testing, using a total test capacity of 2268 plots, 
   3 finally selected genotypes, variance component ratios of 1 : 0.189 : 0.103 : 0.603 : 1.162, 
   and 42 genotypes tested at 9 locations with 2 plot replications at each season across all locations.      
*/
DATA Example1;
INPUT stage_sel total selG Vg Vgxl Vgxs Vgxlxs Verror G_st1 G_st2 G_st3 L_st1 L_st2 L_st3 Rep_st1 Rep_st2 Rep_st3;
CARDS;
1 2268 3 1 0.189 0.103 0.603 1.162 42 42 42 9 9 9 2 2 2
RUN;

/*load macro - in case you follwed our suggestion and files are in desktop, folder SAS_MACROS, subfolder SSG1to3 */
%include "C:\Users\WGRUNEBERG\OneDrive - CGIAR\Desktop\SAS_MACROS\SSG1to3\macroSSG.sas";
/* adapt C:\Users\WGRUNEBERG\OneDrive - CGIAR to location of desktop on you computer                              */  

/*run macro on Example1*/
%SSG1to3(Example1)

/* Example 2: SSG1to3_SAS with input 1 2268 3 1 0.189 0.103 0.603 1.162 63 63 . 9 9 . 2 2 . 
   A one stage selection after two seasons of testing, using a total test capacity of 2268 plots, 
   3 finally selected genotypes, variance component ratios of 1 : 0.189 : 0.103 : 0.603 : 1.162, 
   and 63 genotypes tested at 9 locations with 2 plot replications at both seasons across all locations.
*/ 
DATA Example2;
INPUT stage_sel total selG Vg Vgxl Vgxs Vgxlxs Verror G_st1 G_st2 G_st3 L_st1 L_st2 L_st3 Rep_st1 Rep_st2 Rep_st3;
CARDS;
1 2268 3 1 0.189 0.103 0.603 1.162 63 63 . 9 9 . 2 2 .
RUN;

%include "C:\Users\WGRUNEBERG\OneDrive - CGIAR\Desktop\SAS_MACROS\SSG1to3\macroSSG.sas";
%SSG1to3(Example2)

/* Example 3: SSG1to3_SAS with input 1 2268 3 1 0.189 0.103 0.603 1.162 126 . . 9 . . 2 . .
   A one stage selection after one seasons of testing, using a total test capacity of 2268 plots, 
   3 finally selected genotypes, variance component ratios of 1 : 0.189 : 0.103 : 0.603 : 1.162, 
   and 126 genotypes tested at 9 locations, and 2 plot replications in a single season across all locations.
*/ 
DATA Example3;
INPUT stage_sel total selG Vg Vgxl Vgxs Vgxlxs Verror G_st1 G_st2 G_st3 L_st1 L_st2 L_st3 Rep_st1 Rep_st2 Rep_st3;
CARDS;
1 2268 3 1 0.189 0.103 0.603 1.162 126 . . 9 . . 2 . .
RUN;

%include "C:\Users\WGRUNEBERG\OneDrive - CGIAR\Desktop\SAS_MACROS\SSG1to3\macroSSG.sas";
%SSG1to3(Example3)

/* Example 4: SSG1to3_SAS with input 2 540 3 1 0.189 0.103 0.603 1.162 144 9 . 3 6 . 1 2 . 
   A two stage selection after two seasons of testing, using a total test capacity of 540 plots, 
   3 finally selected genotypes, variance component ratios of 1 : 0.189 : 0.103 : 0.603 : 1.162, 
   and 144 and 9 genotypes tested at the 1st and 2nd selection stage with 3 and 6 locations, respectively,
   and 1 and 2 plot replications at 1st and 2nd selection stage across all locations, respectively.
*/
DATA Example4;
INPUT stage_sel total selG Vg Vgxl Vgxs Vgxlxs Verror G_st1 G_st2 G_st3 L_st1 L_st2 L_st3 Rep_st1 Rep_st2 Rep_st3;
CARDS;
2 540 3 1 0.189 0.103 0.603 1.162 144 9 . 3 6 . 1 2 .
RUN;

%include "C:\Users\WGRUNEBERG\OneDrive - CGIAR\Desktop\SAS_MACROS\SSG1to3\macroSSG.sas";
%SSG1to3(Example4)

/* Example 5: SSG1to3_SAS with input 3 2268 3 1 0.189 0.103 0.603 1.162 499 77 8 1 7 12 2 2 2
   A three stage selection after three seasons of testing, using a total test capacity of 2268 plots, 
   3 finally selected genotypes, variance component ratios of 1 : 0.189 : 0.103 : 0.603 : 1.162, 
   and 499, 77, and 8 genotypes tested at the 1st, 2nd, and 3rd selection stage with 1, 7, and 12 locations,
   respectively, and 2 plot replications at each season across all locations.
*/ 
DATA Example5;
INPUT stage_sel total selG Vg Vgxl Vgxs Vgxlxs Verror G_st1 G_st2 G_st3 L_st1 L_st2 L_st3 Rep_st1 Rep_st2 Rep_st3;
CARDS;
3 2268 3 1 0.189 0.103 0.603 1.162 499 77 8 1 7 12 2 2 2
RUN;

%include "C:\Users\WGRUNEBERG\OneDrive - CGIAR\Desktop\SAS_MACROS\SSG1to3\macroSSG.sas";
%SSG1to3(Example5)

/* Example 6: SSG1to3_SAS with input 3 540 3 1 0.189 0.103 0.603 1.162 138 11 5 3 3 6 1 2 2 
   A three stage selection after three seasons of testing, using a total test capacity of 540 plots, 
   3 finally selected genotypes, variance component ratios of 1 : 0.189 : 0.103 : 0.603 : 1.162, 
   and 138, 11, and 5 genotypes tested at the 1st, 2nd, and 3rd selection stage with 3, 3, and 6 locations, respectively, 
   and 1 plot replication and 2 plot replications at the 1st, 2nd and 3rd selection stage
   across all locations, respectively.      
*/
DATA Example6;
INPUT stage_sel total selG Vg Vgxl Vgxs Vgxlxs Verror G_st1 G_st2 G_st3 L_st1 L_st2 L_st3 Rep_st1 Rep_st2 Rep_st3;
CARDS;
3 540 3 1 0.189 0.103 0.603 1.162 138 11 5 3 3 6 1 2 2
RUN;

%include "C:\Users\WGRUNEBERG\OneDrive - CGIAR\Desktop\SAS_MACROS\SSG1to3\macroSSG.sas";
%SSG1to3(Example6)

/* Example 7: SSG1to3_SAS with input 2 2268 3 1 0.189 0.103 0.603 1.162 500 32 . 3 12 . 1 2 .
   A two stage selection after two seasons of testing, using a total test capacity of 2268 plots, 
   3 finally selected genotypes, variance component ratios of 1 : 0.189 : 0.103 : 0.603 : 1.162, 
   and 500 and 32 genotypes tested at the 1st and 2nd selection stage with 3 and 12 locations, respectively, 
   and 1 plot replication and 2 plot replications at the 1st and 2nd selection stage across all locations, respectively.
*/ 
DATA Example7;
INPUT stage_sel total selG Vg Vgxl Vgxs Vgxlxs Verror G_st1 G_st2 G_st3 L_st1 L_st2 L_st3 Rep_st1 Rep_st2 Rep_st3;
CARDS;
2 2268 3 1 0.189 0.103 0.603 1.162 500 32 . 3 12 . 1 2 .
RUN;

%include "C:\Users\WGRUNEBERG\OneDrive - CGIAR\Desktop\SAS_MACROS\SSG1to3\macroSSG.sas";
%SSG1to3(Example7)

/* Example 8: SSG1to3_SAS with input 2 540 3 1 0.189 0.103 0.603 1.162 144 9 . 3 12 . 1 2 .
   A two stage selection after two seasons of testing, using a total test capacity of 540 plots, 
   3 finally selected genotypes, variance component ratios of 1 : 0.189 : 0.103 : 0.603 : 1.162, 
   and 144 and 9 genotypes tested at the 1st and 2nd selection stage with 3 and 6 locations, respectively, 
   and 1 plot replication and 2 plot replications at the 1st and 2nd selection stage, respectively, across 
   all locations.
 */
DATA Example8;
INPUT stage_sel total selG Vg Vgxl Vgxs Vgxlxs Verror G_st1 G_st2 G_st3 L_st1 L_st2 L_st3 Rep_st1 Rep_st2 Rep_st3;
CARDS;
2 540 3 1 0.189 0.103 0.603 1.162 144 9 . 3 6 . 1 2 .
RUN;

%include "C:\Users\WGRUNEBERG\OneDrive - CGIAR\Desktop\SAS_MACROS\SSG1to3\macroSSG.sas";
%SSG1to3(Example8)

